package Zaphod::Editor::Action::InsertChar;
use strict;
use base qw'Zaphod::Editor::Action';

use Data::Dumper;

use vars qw($VERSION);

use vars qw($VERSION);
$VERSION = '0.02';

__PACKAGE__->verbs([qw(
    insert_char_right delete_char_left
    insert_char_left  delete_char_right
)]);

sub leftmost_status {
    my ($self,$s1,$s2) = @_;

    $s1 ||= $self->status_before;
    $s2 ||= $self->status_after;

    $s1->column < $s2->column
    ? $s1
    : $s2
};

=head2 C<< insert_char_right CHAR >>

Inserts one (or more) characters and moves the cursor
to the right.

=cut

sub insert_char_right {
    my ($class,$editor,$chars) = @_;
    my $status = $editor->status->copy;
    my $status_after = $status->move_right( length $chars );
    
    $class->new(
        offset => $status->offset,
        editor => $editor, 
        status_before => $status, 
        status_after => $status_after, 
        text_before => '', 
        text_after => $chars
    )
};

=head2 C<< insert_char_left CHAR >>

Inserts one (or more) characters and keeps the cursor
where it is.

=cut

sub insert_char_left {
    my ($class,$editor,$chars) = @_;
    my $status = $editor->status->copy;
    my $status_after = $status->copy;

    $class->new(
        offset => $status->offset,
        editor => $editor, 
        status_before => $status, 
        status_after => $status_after, 
        text_before => '', 
        text_after => $chars
    )
};

=head2 C<< delete_char_right COUNT >>

Deletes one (or more) characters to the right of the cursor
and keeps the cursor where it is.

=cut

sub delete_char_right {
    my ($class,$editor,$count) = @_;
    my $status = $editor->status->copy;
    my $status_after = $status->copy;
    
    $count = 1 unless defined $count;
    
    # Needs to know about multichar newlines, maybe?
    my $chars = substr ${$editor->docref}, $status->offset, $count;

    $class->new(
        offset => $status->offset,
        editor => $editor, 
        status_before => $status, 
        status_after => $status_after, 
        text_before => $chars, 
        text_after => '',
    )
};

=head2 C<< delete_char_left COUNT >>

Deletes COUNT characters to the left of the cursor
and moves the cursor to the left.

=cut

sub delete_char_left {
    my ($class,$editor,$count) = @_;
    my $status = $editor->status->copy;
    $count = 1 unless defined $count;
    
    my $status_after = $status->move_left($count);

    # Needs to know about multichar newlines, maybe?
    my $chars = substr ${$editor->docref},$status_after->offset, $count;
    
    $class->new(
        offset => $status->offset,
        editor => $editor, 
        status_before => $status, 
        status_after => $status_after, 
        text_before => $chars, 
        text_after => '',
    )
};

=head1 TODO

=over 4

=item *

Reimplement all *_left as move_left -> *_right

=item *

Think about how to handle multichar newlines

=item *

Implement as_string()

=back

=cut

1;

=head1 AUTHOR

Written by Max Maischein E<lt>corion@cpan.orgE<gt>

Copyright Max Maischein 2005,2006; All Rights Reserved.

This program is free software; you can redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

L<Zaphod>

=cut
